/*
    (C) 2000 AROS - The Amiga Research OS
    $Id: makelink.c,v 1.1.1.1 2005/03/15 16:00:59 laire Exp $
 
    Desc: 
    Lang: English
*/

/******************************************************************************
 
 
    NAME
 
        MakeLink
 
    SYNOPSIS
 
        FROM/A, TO/A, HARD/S, FORCE/S
 
    LOCATION
 
        Workbench:C
 
    FUNCTION
 
        Create a link to a file
 
    INPUTS
 
        FROM   --  The name of the link
	TO     --  The name of the file or directory to link to
	HARD   --  If specified, the link will be a hard-link; default is
	           to create a soft-link
	FORCE  --  Allow a hard-link to point to a directory
 
    RESULT
 
    NOTES
 
    EXAMPLE
 
    BUGS
 
    SEE ALSO
 
    INTERNALS
 
    HISTORY
 
    04.05.2000  SDuvan  implemented
 
******************************************************************************/

#include <exec/memory.h>
#include <dos/dos.h>
#include <dos/dosextens.h>
#include <dos/exall.h>
#include <dos/rdargs.h>
#include <dos/stdio.h>
#include <utility/tagitem.h>

#include <proto/dos.h>
#include <proto/exec.h>

#include "makelink_version.h"

static const char version[] = VERSTAG;
static const char progname[] = "MakeLink";

enum { ARG_FROM = 0, ARG_TO, ARG_HARD, ARG_FORCE };

int main(void)
{
    struct Library *SysBase = *((struct Library **) 4);
    struct DosLibrary *DOSBase;
    int  retval = RETURN_FAIL;

    DOSBase = (struct DosLibrary *) OpenLibrary("dos.library", 37L);

    if (DOSBase)
    {
        struct RDArgs *rda;
        IPTR args[] = { NULL, NULL, (IPTR)FALSE, (IPTR)FALSE };

        rda = ReadArgs("FROM/A,TO/A,HARD/S,FORCE/S", args, NULL);

        if (rda)
        {
            BPTR  lock;

            lock = Lock((STRPTR) args[ARG_TO], SHARED_LOCK);

            #warning "TODO: Add softlink src relativity stuff"

            if (lock)
            {
                struct FileInfoBlock *fib = AllocDosObject(DOS_FIB, NULL);

                if (fib)
                {
                    if (Examine(lock, fib))
                    {
                        /* Directories may only be hard-linked to if FORCE is
                           specified */
                        if (fib->fib_DirEntryType >= 0 &&
                            !args[ARG_FORCE] && args[ARG_HARD])
                        {
                            PutStr("Hard-links to directories require the FORCE "
                                   "keyword\n");
                        }
                        else
                        {
                            #warning "FIXME: Add check for circular links!"

                            if (MakeLink((STRPTR) args[ARG_FROM],
                                         args[ARG_HARD] ? (LONG) lock : (LONG) args[ARG_TO],
                                         !args[ARG_HARD]))
                            {
                                retval = RETURN_OK;
                            }
                            else
                            {
                                PrintFault(IoErr(), progname);
                            }
                        }
                    }

                    FreeDosObject(DOS_FIB, fib);
                }
                else
                {
                    PrintFault(IoErr(), progname);
                }

                UnLock(lock);
            }
            else
            {
                PutStr((STRPTR)args[ARG_TO]);
                PrintFault(IoErr(), "");
            }

            FreeArgs(rda);
        }
        else
        {
            PrintFault(IoErr(), progname);
        }

        CloseLibrary((struct Library *) DOSBase);
    }

    return retval;
}
